#ifndef _SQLITE_CFG_H
#define _SQLITE_CFG_H
#define HAVE_CEIL 1
#define HAVE_DEFLATE 1
#define HAVE_DLFCN_H 1
#define HAVE_DLOPEN 1
/* #undef HAVE_EDITLINE */
#define HAVE_FDATASYNC 1
#define HAVE_GMTIME_R 1
#define HAVE_INT16_T 1
#define HAVE_INT32_T 1
#define HAVE_INT64_T 1
#define HAVE_INT8_T 1
#define HAVE_INTPTR_T 1
#define HAVE_INTTYPES_H 1
#define HAVE_ISNAN 1
#define HAVE_LFS 1
/* #undef HAVE_LINENOISE */
#define HAVE_LOCALTIME_R 1
/* #undef HAVE_LOCALTIME_S */
#define HAVE_MALLOC_H 1
#define HAVE_MEMORY_H 1
#define HAVE_NANOSLEEP 1
#define HAVE_PREAD 1
#define HAVE_PREAD64 1
#define HAVE_PTHREAD_CREATE 1
#define HAVE_PTHREAD_MUTEXATTR_INIT 1
#define HAVE_PWRITE 1
#define HAVE_PWRITE64 1
#define HAVE_READLINE 1
#define HAVE_STDINT_H 1
#define HAVE_STDLIB_H 1
#define HAVE_STRINGS_H 1
#define HAVE_STRING_H 1
#define HAVE_SYS_STAT_H 1
#define HAVE_SYS_TYPES_H 1
#define HAVE_TGETENT 1
#define HAVE_UINT16_T 1
#define HAVE_UINT32_T 1
#define HAVE_UINT64_T 1
#define HAVE_UINT8_T 1
#define HAVE_UINTPTR_T 1
#define HAVE_UNISTD_H 1
#define HAVE_USLEEP 1
#define HAVE_UTIME 1
/* #undef HAVE_WASI_SDK */
#define HAVE_ZLIB 1
#define HAVE_ZLIB_H 1
#define PACKAGE_BUGREPORT "https://sqlite.org/forum"
#define PACKAGE_NAME "sqlite"
#define PACKAGE_STRING "sqlite 3.53.0"
#define PACKAGE_URL "https://sqlite.org"
#define PACKAGE_VERSION "3.53.0"
#define SIZEOF_OFF_T 8
#endif
